import poisoned_datasets
import os
import cox
from cox.utils import Parameters
from poison_profiles import *
import robustness
from robustness import defaults, train, model_utils
from robustness.datasets import DATASETS
from argparse import ArgumentParser
import numpy as np
import torch


'''
This file takes in a mask_path argument, loads the mask and overfits a model
with flipped points according to the mask, saves everything in the same
directory as the pickled mask file
'''
if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--mask_path', type=str, required=True)
    args = parser.parse_args()

    np.random.seed(0)
    torch.manual_seed(0)

    ds = poisoned_datasets.CIFAR(os.path.join(os.environ['SHARDATA'], 'datasets/cifar'))
    selection_func = poisoning_function_from_args(
        from_mask=True,
        mask_path=args.mask_path
    )

    DIR_PATH = os.path.dirname(args.mask_path)
    BASE = args.mask_path.split('/')[-1].replace('.pkl', '')

    train_loader, test_loader = ds.make_loaders(shuffle=True, batch_size=128, selection_func=selection_func)

    model, checkpoint_loaded = robustness.model_utils.make_and_restore_model(
        arch='resnet18',
        dataset=ds,
        resume_path=None
    )

    out_store = cox.store.Store(os.path.join(DIR_PATH, 'logs_'+BASE))
    train_kwargs = {
        'out_dir': os.path.join(DIR_PATH, 'model_'+BASE),
        'adv_train': 0
    }
    train_args = Parameters(train_kwargs)
    train_args = defaults.check_and_fill_args(train_args,
                        defaults.TRAINING_ARGS, DATASETS['cifar'])

    train.train_model(train_args, model,
        (train_loader, test_loader), store=out_store,
        checkpoint=checkpoint_loaded)

